/* eslint-disable */
//	Stores form field values and validations
const Store = function (defaultValues, valuesObserver, validationsObserver) {
	var me = {
		values: {},
		validations: {},
		fieldRefs: {},
		setFieldRef(ref) {
			const name = ref.props && ref.props.name
			const arrayIndex = ref.props && ref.props.arrayIndex
			if (name) {
				if (arrayIndex !== undefined) {
					me.fieldRefs[name] = me.fieldRefs[name] || []
					me.fieldRefs[name][arrayIndex] = ref
				} else {
					me.fieldRefs[name] = ref
				}
			}
		},
		deleteFieldRef(ref) {
			const name = ref.props && ref.props.name
			const arrayIndex = ref.props && ref.props.arrayIndex
			if (name) {
				if (arrayIndex !== undefined) {
					delete me.fieldRefs[name][arrayIndex]
					if (!me.fieldRefs[name].length) {
						delete me.fieldRefs[name]
					}
				} else {
					delete me.fieldRefs[name]
				}
			}
		},
		//	Clear the store - optionally skip the observer
		clear(skipObserver) {
			me.values = {}
			if (!skipObserver && valuesObserver) {
				valuesObserver(me.values)
			}
		},
		//	Sets a value - or merge an object's values
		set(key, value, arrayIndex) {
			if (typeof key === 'object') {
				for (const i in key) {
					if (key.hasOwnProperty(i)) {
						me.values[i] = key[i]
					}
				}
			} else {
				if(arrayIndex !== undefined) {
					me.values[key] = me.values[key] || []
					me.values[key][arrayIndex] = value
				} else {
					me.values[key] = value
				}
			}
			if (valuesObserver) {
				valuesObserver(me.values)
			}
		},
		//	Gets a value, or array index
		get(key, arrayIndex, allowEmpty) {
			let value = key ?
				me.values[key] :
				me.values
			if(arrayIndex !== undefined) {
				value = value[arrayIndex]
			}
			//	Ensure we don't get the dreaded:
			//	"...is changing a controlled input of type text to be uncontrolled." warning.
			//	TODO: Make this configurable
			return value !== undefined ? value : (allowEmpty ? value : '');
		},
		//	Binders for setting the value - use for onChange events that are outside of the
		//	field, such as RadioButtonGroup in material-ui
		setBinder(name) {
			return function (e, value) {
				let setValue = (e !== undefined && e.target && e.target.value) ?
					e.target.value : value
				setValue = setValue !== undefined ? setValue : ''
				me.set(name, setValue)
			}
		},
		getBinder(name) {
			return function () {
				return me.get(name)
			}
		},

		//	Validations
		setValidations(validations) {
			Object.keys(validations).forEach(key => me.validations[key] = validations[key])
			if (validationsObserver) {
				const validationResult = Object.keys(me.validations).every(key => !me.validations[key])
				validationsObserver(me.validations, validationResult)
			}
		},
		setValidation(key, value) {
			me.validations[key] = value
			if (validationsObserver) {
				const validationResult = Object.keys(me.validations).every(key => !me.validations[key])
				validationsObserver(me.validations, validationResult)
			}
		},
		getValidation(key) {
			return me.validations[key]
		}
	}

	if (defaultValues) {
		me.set(defaultValues)
	}

	return me
}

export default Store
