import cache from '../utilities/cache'
import debounce from '../utilities/debounce'

const validators = {}

const validatorFor = (options, key) => {
	if (!options.debounce) {
		return options.validate
	}

	const validatorKey = `key:${key}_function-name:${options.validate.name}`
	const validator = validators[validatorKey]
	if (validator && validator.options.validate === options.validate) {
		return validator.validate
	}

	validators[validatorKey] = {
		options,
		validate: debounce(options.validate, options.debounce.duration)
	}
	return validators[validatorKey].validate
}

const asyncValidator = async (value, options, key) => {
	if (typeof options.validate === 'function') {
		const validator = validatorFor(options, key)

		try {
			if (options.cache) {
				return await cache(validator, key, value, options.cache.duration)
			}
			return await validator(value)
		} catch (rejection) {
			// in debounce mode, promise rejections are to be ignored
			if (options.debounce) {
				return undefined
			}
			throw rejection
		}
	}
	return undefined
}

export default asyncValidator
